==> cas_server-2.0.0.tar.gz: cas_server/src/Controller/UserActionController.php <==
<?php

namespace Drupal\cas_server\Controller;

use Drupal\cas_server\Configuration\ConfigHelper;
use Drupal\cas_server\EventSubscriber\CASResponseCookies;
use Drupal\cas_server\Exception\TicketMissingException;
use Drupal\cas_server\Exception\TicketTypeException;
use Drupal\cas_server\Logger\DebugLogger;
use Drupal\cas_server\RedirectResponse;
use Drupal\cas_server\Ticket\TicketFactory;
use Drupal\cas_server\Ticket\TicketStorageInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Utility\Crypt;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Session\SessionManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Process user input into actions.
 */
class UserActionController implements ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * Used to get the query string parameters from the request.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The current user, or anonymous.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $account;

  /**
   * The configuration helper.
   *
   * @var \Drupal\cas_server\Configuration\ConfigHelper
   */
  protected $configHelper;

  /**
   * The ticket factory.
   *
   * @var \Drupal\cas_server\Ticket\TicketFactory
   */
  protected $ticketFactory;

  /**
   * The ticket storage.
   *
   * @var \Drupal\cas_server\Ticket\TicketStorageInterface
   */
  protected $ticketStore;

  /**
   * The logger.
   *
   * @var \Drupal\cas_server\Logger\DebugLogger
   */
  protected $logger;

  /**
   * The session manager.
   *
   * @var \Drupal\Core\Session\SessionManagerInterface
   */
  protected $sessionManager;

  /**
   * The page cache kill switch.
   *
   * @var \Drupal\Core\PageCache\ResponsePolicy\KillSwitch
   */
  protected $killSwitch;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * Drupal\cas_server\EventSubscriber\CASResponseCookies definition.
   *
   * @var \Drupal\cas_server\EventSubscriber\CASResponseCookies
   */
  protected $responseCookies;

  /**
   * Constructor.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Symfony request stack.
   * @param \Drupal\Core\Session\AccountProxyInterface $user
   *   The current user.
   * @param \Drupal\cas_server\Configuration\ConfigHelper $config_helper
   *   The configuration helper.
   * @param \Drupal\cas_server\Ticket\TicketFactory $ticket_factory
   *   The ticket factory.
   * @param \Drupal\cas_server\Ticket\TicketStorageInterface $ticket_store
   *   The ticket store.
   * @param \Drupal\cas_server\Logger\DebugLogger $debug_logger
   *   The logger.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $translation
   *   The string translation service.
   * @param \Drupal\Core\Session\SessionManagerInterface $session_manager
   *   The session manager.
   * @param \Drupal\Core\PageCache\ResponsePolicy\KillSwitch $kill_switch
   *   The page cache kill switch.
   * @param \Drupal\Component\Datetime\TimeInterface $time_service
   *   The time service.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder service.
   * @param \Drupal\cas_server\EventSubscriber\CASResponseCookies $response_cookies
   *   The returned response cookies manager.
   */
  public function __construct(
    RequestStack $request_stack,
    AccountProxyInterface $user,
    ConfigHelper $config_helper,
    TicketFactory $ticket_factory,
    TicketStorageInterface $ticket_store,
    DebugLogger $debug_logger,
    TranslationInterface $translation,
    SessionManagerInterface $session_manager,
    KillSwitch $kill_switch,
    TimeInterface $time_service,
    FormBuilderInterface $form_builder,
    CASResponseCookies $response_cookies
  ) {
    $this->requestStack = $request_stack;
    $this->account = $user;
    $this->configHelper = $config_helper;
    $this->ticketFactory = $ticket_factory;
    $this->ticketStore = $ticket_store;
    $this->logger = $debug_logger;
    $this->stringTranslation = $translation;
    $this->sessionManager = $session_manager;
    $this->killSwitch = $kill_switch;
    $this->time = $time_service;
    $this->formBuilder = $form_builder;
    $this->responseCookies = $response_cookies;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('request_stack'),
      $container->get('current_user'),
      $container->get('cas_server.config_helper'),
      $container->get('cas_server.ticket_factory'),
      $container->get('cas_server.storage'),
      $container->get('cas_server.logger'),
      $container->get('string_translation'),
      $container->get('session_manager'),
      $container->get('page_cache_kill_switch'),
      $container->get('datetime.time'),
      $container->get('form_builder'),
      $container->get('cas_server.cas_response_cookies_subscriber')
    );
  }

  /**
   * Handles a page request for /cas/login.
   */
  public function login() {
    $this->killSwitch->trigger();
    $request = $this->requestStack->getCurrentRequest();
    $service = $request->query->has('service') ? $request->query->get('service') : NULL;

    // If we have a ticket, it is because we've already processed the form and
    // need to be redirected back to the service.
    if ($request->query->has('ticket') && $this->configHelper->checkServiceAgainstWhitelist($service)) {
      $url = Url::fromUri($service, ['query' => ['ticket' => $request->query->get('ticket')]]);
      $response = new TrustedRedirectResponse($url->toString(), 302);
      $response
        ->addCacheableDependency((new CacheableMetadata())->setCacheMaxAge(0));
      return $response;
    }

    if ($request->query->has('gateway')) {
      if ($request->query->get('gateway') == 'false') {
        $gateway = FALSE;
      }
      else {
        $gateway = (bool) $request->query->get('gateway');
      }
    }
    else {
      $gateway = FALSE;
    }

    if ($request->query->has('renew')) {
      if ($request->query->get('renew') == 'false') {
        $renew = FALSE;
      }
      else {
        $renew = (bool) $request->query->get('renew');
      }
    }
    else {
      $renew = FALSE;
    }

    // If no service, need to either show the login form (if not logged in),
    // or a simple page to logged in users explaining their state.
    if (is_null($service)) {
      if (!$this->userHasSingleSignOnSession(NULL)) {
        return $this->formBuilder->getForm(
          '\Drupal\cas_server\Form\UserLogin',
          ''
        );
      }
      else {
        return $this->generateLoggedInMessage();
      }
    }

    // Check service against whitelist. If its not a valid service, display
    // a page to that effect.
    $serviceEntity = $this->configHelper->loadServiceFromUri($service);
    if (!$serviceEntity) {
      return $this->generateInvalidServiceMessage();
    }

    // Check if authenticated user is allowed to use this service. Anon user
    // can attempt to log into any service. A returning auth user will be
    // checked to see if they are permitted before granting cookie.
    if ($this->account->isAuthenticated()) {
      if (!$serviceEntity->accountPermitted($this->account)) {
        return $this->generateNotPermittedMessage();
      }
    }

    // If user has an active single sign on session and renew is not set,
    // generate a service ticket and redirect.
    if (!$renew && $this->userHasSingleSignOnSession($service)) {
      $st = $this->ticketFactory->createServiceTicket($service, FALSE);
      $url = Url::fromUri($service, ['query' => ['ticket' => $st->getId()]]);
      $response = new TrustedRedirectResponse($url->toString(), 302);
      $metadata = (new CacheableMetadata())->setCacheMaxAge(0);
      $response->addCacheableDependency($metadata);
      return $response;
    }

    // If gateway is set and user is not logged in, redirect them back to
    // service.
    if ($gateway && !$this->userHasSingleSignOnSession($service)) {
      $response = new TrustedRedirectResponse($service, 302);
      $metadata = (new CacheableMetadata())->setCacheMaxAge(0);
      $response->addCacheableDependency($metadata);
      return $response;
    }

    // Present the user with a login form.
    return $this->formBuilder->getForm(
      '\Drupal\cas_server\Form\UserLogin',
      $service
    );
  }

  /**
   * Handles a page request for /cas/logout.
   */
  public function logout() {
    $this->killSwitch->trigger();
    $request = $this->requestStack->getCurrentRequest();
    $service = $request->query->has('service')
      ? $request->query->get('service')
      : NULL;
    if ($request->cookies->has('cas_tgc')) {
      // Remove the ticket granting cookie.
      $this->responseCookies->clearCookie(
        'cas_tgc',
        Url::fromUri('internal:/cas')->toString()
      );
    }
    $session_id = $this->sessionManager->getId();
    $hashed_id = Crypt::hashBase64($session_id);
    $this->ticketStore->deleteTicketsBySession($hashed_id);
    $this->userLogout();

    // If we have a ticket, it is because we've already processed the form and
    // need to be redirected back to the service.
    if ($service) {
      return new RedirectResponse($service, 302);
    }

    return $this->generateUserLogoutPage();
  }

  /**
   * User has a valid single sign on session for a given service.
   *
   * @param string $service
   *   The service to check for.
   *
   * @return bool
   *   Return TRUE when a user has a valid SSO session.
   */
  private function userHasSingleSignOnSession($service) {
    if (!is_null($service) && !$this->configHelper->verifyServiceForSso($service)) {
      return FALSE;
    }

    $should_use_tgc = $this->configHelper->shouldUseTicketGrantingTicket();
    $request = $this->requestStack->getCurrentRequest();
    if ($should_use_tgc && $request->cookies->has('cas_tgc')) {
      $cas_tgc = urldecode($request->cookies->get('cas_tgc'));
      try {
        $tgt = $this->ticketStore->retrieveTicketGrantingTicket($cas_tgc);
      }
      catch (TicketTypeException $e) {
        $this->logger->log('Bad ticket type: @error_msg', [
          '@error_msg' => $e->getMessage(),
        ]);
        return FALSE;
      }
      catch (TicketMissingException $e) {
        $this->logger->log('Ticket not found: @ticket', [
          '@ticket' => $cas_tgc,
        ]);
        return FALSE;
      }

      if ($this->time->getRequestTime() > $tgt->getExpirationTime()) {
        $this->ticketStore->deleteTicketGrantingTicket($tgt);
        return FALSE;
      }

      if ($this->account->id() != $tgt->getUid()) {
        return FALSE;
      }

      return TRUE;
    }
    elseif (!$should_use_tgc && !$this->account->isAnonymous()) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Markup for an invalid service message.
   *
   * @return array
   *   A renderable array.
   */
  private function generateInvalidServiceMessage() {
    $m = $this->configHelper->getInvalidServiceMessage();
    $message = $m ? $m : $this->t('You have not requested a valid service.');

    return [
      'wrapper' => [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['cas-message-box', 'cas-message-invalid-service'],
        ],

        'header' => [
          '#prefix' => '<h2>',
          '#suffix' => '</h2>',
          '#markup' => $this->t('Invalid Service'),
        ],

        'message_wrapper' => [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['cas-message'],
          ],
          'message' => [
            '#markup' => $message,
          ],
        ],
      ],
    ];
  }

  /**
   * Markup for a not permitted message.
   *
   * @return array
   *   A renderable array.
   */
  private function generateNotPermittedMessage() {
    $m = $this->configHelper->getNotPermittedMessage();
    $message = $m ? $m : $this->t('Your account does not have the required permissions to log in to this service.');

    return [
      'wrapper' => [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['cas-message-box', 'cas-message-access-denied'],
        ],

        'header' => [
          '#prefix' => '<h2>',
          '#suffix' => '</h2>',
          '#markup' => $this->t('Access denied'),
        ],

        'message_wrapper' => [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['cas-message'],
          ],

          'message' => [
            '#markup' => $message,
          ],
        ],
      ],
    ];
  }

  /**
   * Markup for logout message.
   *
   * @return array
   *   A renderable array.
   */
  protected function generateUserLogoutPage() {
    $m = $this->configHelper->getUserLogoutMessage();
    $message = $m ? $m : $this->t('You have been logged out');

    return [
      'wrapper' => [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['cas-message-box', 'cas-message-logout'],
        ],

        'message_wrapper' => [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['cas-message'],
          ],

          'message' => [
            '#markup' => $message,
          ],
        ],
      ],
    ];
  }

  /**
   * Markup for logged in message.
   *
   * @return array
   *   A renderable array.
   */
  protected function generateLoggedInMessage() {
    $m = $this->configHelper->getLoggedInMessage();
    $message = $m ? $m : $this->t('You are logged in to CAS single sign on.');

    return [
      'wrapper' => [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['cas-message-box', 'cas-message-login'],
        ],

        'message_wrapper' => [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['cas-message'],
          ],

          'message' => [
            '#markup' => $message,
          ],
        ],
      ],
    ];
  }

  /**
   * Encapsulates user_logout.
   */
  private function userLogout() {
    if ($this->account->isAuthenticated()) {
      user_logout();
    }
  }

}

==> cas_server-2.0.1.tar.gz: cas_server/src/Controller/UserActionController.php <==
<?php

namespace Drupal\cas_server\Controller;

use Drupal\cas_server\Configuration\ConfigHelper;
use Drupal\cas_server\EventSubscriber\CASResponseCookies;
use Drupal\cas_server\Exception\TicketMissingException;
use Drupal\cas_server\Exception\TicketTypeException;
use Drupal\cas_server\Logger\DebugLogger;
use Drupal\cas_server\RedirectResponse;
use Drupal\cas_server\Ticket\TicketFactory;
use Drupal\cas_server\Ticket\TicketStorageInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Process user input into actions.
 */
class UserActionController implements ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * Used to get the query string parameters from the request.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The current user, or anonymous.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $account;

  /**
   * The configuration helper.
   *
   * @var \Drupal\cas_server\Configuration\ConfigHelper
   */
  protected $configHelper;

  /**
   * The ticket factory.
   *
   * @var \Drupal\cas_server\Ticket\TicketFactory
   */
  protected $ticketFactory;

  /**
   * The ticket storage.
   *
   * @var \Drupal\cas_server\Ticket\TicketStorageInterface
   */
  protected $ticketStore;

  /**
   * The logger.
   *
   * @var \Drupal\cas_server\Logger\DebugLogger
   */
  protected $logger;

  /**
   * The page cache kill switch.
   *
   * @var \Drupal\Core\PageCache\ResponsePolicy\KillSwitch
   */
  protected $killSwitch;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * Drupal\cas_server\EventSubscriber\CASResponseCookies definition.
   *
   * @var \Drupal\cas_server\EventSubscriber\CASResponseCookies
   */
  protected $responseCookies;

  /**
   * Constructor.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Symfony request stack.
   * @param \Drupal\Core\Session\AccountProxyInterface $user
   *   The current user.
   * @param \Drupal\cas_server\Configuration\ConfigHelper $config_helper
   *   The configuration helper.
   * @param \Drupal\cas_server\Ticket\TicketFactory $ticket_factory
   *   The ticket factory.
   * @param \Drupal\cas_server\Ticket\TicketStorageInterface $ticket_store
   *   The ticket store.
   * @param \Drupal\cas_server\Logger\DebugLogger $debug_logger
   *   The logger.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $translation
   *   The string translation service.
   * @param \Drupal\Core\PageCache\ResponsePolicy\KillSwitch $kill_switch
   *   The page cache kill switch.
   * @param \Drupal\Component\Datetime\TimeInterface $time_service
   *   The time service.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder service.
   * @param \Drupal\cas_server\EventSubscriber\CASResponseCookies $response_cookies
   *   The returned response cookies manager.
   *
   * @phpcs:disable Drupal.Functions.MultiLineFunctionDeclaration.MissingTrailingComma
   */
  public function __construct(
    RequestStack $request_stack,
    AccountProxyInterface $user,
    ConfigHelper $config_helper,
    TicketFactory $ticket_factory,
    TicketStorageInterface $ticket_store,
    DebugLogger $debug_logger,
    TranslationInterface $translation,
    KillSwitch $kill_switch,
    TimeInterface $time_service,
    FormBuilderInterface $form_builder,
    CASResponseCookies $response_cookies
  ) {
    // phpcs:enable
    $this->requestStack = $request_stack;
    $this->account = $user;
    $this->configHelper = $config_helper;
    $this->ticketFactory = $ticket_factory;
    $this->ticketStore = $ticket_store;
    $this->logger = $debug_logger;
    $this->stringTranslation = $translation;
    $this->killSwitch = $kill_switch;
    $this->time = $time_service;
    $this->formBuilder = $form_builder;
    $this->responseCookies = $response_cookies;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('request_stack'),
      $container->get('current_user'),
      $container->get('cas_server.config_helper'),
      $container->get('cas_server.ticket_factory'),
      $container->get('cas_server.storage'),
      $container->get('cas_server.logger'),
      $container->get('string_translation'),
      $container->get('page_cache_kill_switch'),
      $container->get('datetime.time'),
      $container->get('form_builder'),
      $container->get('cas_server.cas_response_cookies_subscriber')
    );
  }

  /**
   * Handles a page request for /cas/login.
   */
  public function login() {
    $this->killSwitch->trigger();
    $request = $this->requestStack->getCurrentRequest();
    $service = $request->query->has('service') ? $request->query->get('service') : NULL;

    // If we have a ticket, it is because we've already processed the form and
    // need to be redirected back to the service.
    if ($request->query->has('ticket') && $this->configHelper->checkServiceAgainstWhitelist($service)) {
      $url = Url::fromUri($service, ['query' => ['ticket' => $request->query->get('ticket')]]);
      $response = new TrustedRedirectResponse($url->toString(), 302);
      $response
        ->addCacheableDependency((new CacheableMetadata())->setCacheMaxAge(0));
      return $response;
    }

    if ($request->query->has('gateway')) {
      if ($request->query->get('gateway') == 'false') {
        $gateway = FALSE;
      }
      else {
        $gateway = (bool) $request->query->get('gateway');
      }
    }
    else {
      $gateway = FALSE;
    }

    if ($request->query->has('renew')) {
      if ($request->query->get('renew') == 'false') {
        $renew = FALSE;
      }
      else {
        $renew = (bool) $request->query->get('renew');
      }
    }
    else {
      $renew = FALSE;
    }

    // If no service, need to either show the login form (if not logged in),
    // or a simple page to logged in users explaining their state.
    if (is_null($service)) {
      if (!$this->userHasSingleSignOnSession(NULL)) {
        return $this->formBuilder->getForm(
          '\Drupal\cas_server\Form\UserLogin',
          ''
        );
      }
      else {
        return $this->generateLoggedInMessage();
      }
    }

    // Check service against whitelist. If its not a valid service, display
    // a page to that effect.
    $serviceEntity = $this->configHelper->loadServiceFromUri($service);
    if (!$serviceEntity) {
      return $this->generateInvalidServiceMessage();
    }

    // Check if authenticated user is allowed to use this service. Anon user
    // can attempt to log into any service. A returning auth user will be
    // checked to see if they are permitted before granting cookie.
    if ($this->account->isAuthenticated()) {
      if (!$serviceEntity->accountPermitted($this->account)) {
        return $this->generateNotPermittedMessage();
      }
    }

    // If user has an active single sign on session and renew is not set,
    // generate a service ticket and redirect.
    if (!$renew && $this->userHasSingleSignOnSession($service)) {
      $st = $this->ticketFactory->createServiceTicket($service, FALSE);
      $url = Url::fromUri($service, ['query' => ['ticket' => $st->getId()]]);
      $response = new TrustedRedirectResponse($url->toString(), 302);
      $metadata = (new CacheableMetadata())->setCacheMaxAge(0);
      $response->addCacheableDependency($metadata);
      return $response;
    }

    // If gateway is set and user is not logged in, redirect them back to
    // service.
    if ($gateway && !$this->userHasSingleSignOnSession($service)) {
      $response = new TrustedRedirectResponse($service, 302);
      $metadata = (new CacheableMetadata())->setCacheMaxAge(0);
      $response->addCacheableDependency($metadata);
      return $response;
    }

    // Present the user with a login form.
    return $this->formBuilder->getForm(
      '\Drupal\cas_server\Form\UserLogin',
      $service
    );
  }

  /**
   * Handles a page request for /cas/logout.
   */
  public function logout() {
    $this->killSwitch->trigger();
    $request = $this->requestStack->getCurrentRequest();
    $service = $request->query->has('service')
      ? $request->query->get('service')
      : NULL;
    if ($request->cookies->has('cas_tgc')) {
      // Remove the ticket granting cookie.
      $this->responseCookies->clearCookie(
        'cas_tgc',
        Url::fromUri('internal:/cas')->toString()
      );
    }

    // Delete all tickets related to this session if id is available.
    if ($session_id = $this->ticketFactory->getUniqueId(FALSE)) {
      $this->ticketStore->deleteTicketsBySession($session_id);
    }

    $this->userLogout();

    // If we have a ticket, it is because we've already processed the form and
    // need to be redirected back to the service.
    if ($service) {
      return new RedirectResponse($service, 302);
    }

    return $this->generateUserLogoutPage();
  }

  /**
   * User has a valid single sign on session for a given service.
   *
   * @param string $service
   *   The service to check for.
   *
   * @return bool
   *   Return TRUE when a user has a valid SSO session.
   */
  private function userHasSingleSignOnSession($service) {
    if (!is_null($service) && !$this->configHelper->verifyServiceForSso($service)) {
      return FALSE;
    }

    $should_use_tgc = $this->configHelper->shouldUseTicketGrantingTicket();
    $request = $this->requestStack->getCurrentRequest();
    if ($should_use_tgc && $request->cookies->has('cas_tgc')) {
      $cas_tgc = urldecode($request->cookies->get('cas_tgc'));
      try {
        $tgt = $this->ticketStore->retrieveTicketGrantingTicket($cas_tgc);
      }
      catch (TicketTypeException $e) {
        $this->logger->log('Bad ticket type: @error_msg', [
          '@error_msg' => $e->getMessage(),
        ]);
        return FALSE;
      }
      catch (TicketMissingException $e) {
        $this->logger->log('Ticket not found: @ticket', [
          '@ticket' => $cas_tgc,
        ]);
        return FALSE;
      }

      if ($this->time->getRequestTime() > $tgt->getExpirationTime()) {
        $this->ticketStore->deleteTicketGrantingTicket($tgt);
        return FALSE;
      }

      if ($this->account->id() != $tgt->getUid()) {
        return FALSE;
      }

      return TRUE;
    }
    elseif (!$should_use_tgc && !$this->account->isAnonymous()) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Markup for an invalid service message.
   *
   * @return array
   *   A renderable array.
   */
  private function generateInvalidServiceMessage() {
    $m = $this->configHelper->getInvalidServiceMessage();
    $message = $m ? $m : $this->t('You have not requested a valid service.');

    return [
      'wrapper' => [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['cas-message-box', 'cas-message-invalid-service'],
        ],

        'header' => [
          '#prefix' => '<h2>',
          '#suffix' => '</h2>',
          '#markup' => $this->t('Invalid Service'),
        ],

        'message_wrapper' => [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['cas-message'],
          ],
          'message' => [
            '#markup' => $message,
          ],
        ],
      ],
    ];
  }

  /**
   * Markup for a not permitted message.
   *
   * @return array
   *   A renderable array.
   */
  private function generateNotPermittedMessage() {
    $m = $this->configHelper->getNotPermittedMessage();
    $message = $m ? $m : $this->t('Your account does not have the required permissions to log in to this service.');

    return [
      'wrapper' => [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['cas-message-box', 'cas-message-access-denied'],
        ],

        'header' => [
          '#prefix' => '<h2>',
          '#suffix' => '</h2>',
          '#markup' => $this->t('Access denied'),
        ],

        'message_wrapper' => [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['cas-message'],
          ],

          'message' => [
            '#markup' => $message,
          ],
        ],
      ],
    ];
  }

  /**
   * Markup for logout message.
   *
   * @return array
   *   A renderable array.
   */
  protected function generateUserLogoutPage() {
    $m = $this->configHelper->getUserLogoutMessage();
    $message = $m ? $m : $this->t('You have been logged out');

    return [
      'wrapper' => [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['cas-message-box', 'cas-message-logout'],
        ],

        'message_wrapper' => [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['cas-message'],
          ],

          'message' => [
            '#markup' => $message,
          ],
        ],
      ],
    ];
  }

  /**
   * Markup for logged in message.
   *
   * @return array
   *   A renderable array.
   */
  protected function generateLoggedInMessage() {
    $m = $this->configHelper->getLoggedInMessage();
    $message = $m ? $m : $this->t('You are logged in to CAS single sign on.');

    return [
      'wrapper' => [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['cas-message-box', 'cas-message-login'],
        ],

        'message_wrapper' => [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['cas-message'],
          ],

          'message' => [
            '#markup' => $message,
          ],
        ],
      ],
    ];
  }

  /**
   * Encapsulates user_logout.
   */
  private function userLogout() {
    if ($this->account->isAuthenticated()) {
      user_logout();
    }
  }

}
